#!/usr/bin/env Rscript
# ITI project
#
# Copyright Maxime Garcia, Ghislain Bidaut, Claire Rioualen, Quentin Da Costa
# (2009-2017)
#
# maxime.garcia@inserm.fr
# ghislain.bidaut@inserm.fr
# claire.rioualen@inserm.fr
# quentin.da-costa@inserm.fr
#
# http://iti.sourceforge.net
#
# This software is a computer program whose purpose is to superimpose
# interactome over several gene expression datasets to find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

# Authors:
# Claire Rioualen	claire.rioualen@inserm.fr
# Quentin Da Costa	quentin.da-costa@inserm.fr
# Ghislain Bidaut	ghislain.bidaut@inserm.fr
# Guillaume Tiberi	guillaume.tiberi@inserm.fr

# Version:
# 2014.09

# Example of use:
# ~/workspace/iti/pipeline/R/tableToHeatmap.R inputTable.txt nb_of_conditions outputPicture.png


## Packages ##
library(Heatplus)

args <- commandArgs(TRUE)

inputTable    = args[1]
condNb        = as.numeric(args[2])
outputPicture = args[3]

expDF = read.table(inputTable, header = T, sep = "\t", check.names = F, stringsAsFactors = F)

expAnnotDF = t(expDF[1:condNb,])
expMatrix  = as.matrix(expDF[-1:-condNb,,drop=FALSE])
mode(expMatrix) = "numeric"	# Or: apply(expMatrix, 1:2, as.numeric)

if(condNb == 0) {
    heatmap = annHeatmap2(expMatrix, dendrogram = list(status = "no"), labels = list(cex = 1.5, nrow = 10), legend = T)
} else {
    heatmap = annHeatmap2(expMatrix, dendrogram = list(status = "no"), annotation = list(Col = list(data = expAnnotDF)), labels = list(cex = 1.5, nrow = 10), legend = T)
}

dim    = 500
geneNb = nrow(expMatrix)
if(geneNb > 20){
	dim = geneNb * 25
}


if(capabilities('X11')) {
	png(outputPicture, width = dim, height = dim)
	plot(heatmap)
	trash = dev.off()
}
else {
     warning('No X11 Capabilities - No png nor pdf produced')
}
