#!/usr/bin/perl
#
# ITI project
#
# Copyright Maxime Garcia, Ghislain Bidaut, Claire Rioualen, Quentin Da Costa
# (2009-2017)
#
# maxime.garcia@inserm.fr
# ghislain.bidaut@inserm.fr
# claire.rioualen@inserm.fr
# quentin.da-costa@inserm.fr
#
# http://iti.sourceforge.net
#
# This software is a computer program whose purpose is to superimpose
# interactome over several gene expression datasets to find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package Web;

use strict;
use warnings;
use Utils;

use Metasubnet;
use List::Util;

=pod

=head2 writeCategory()

Main function. Creates the HTML pages of the specified analysis.

=cut

sub writeCategory {
    my $outputDir = shift;
    my $itiPath = shift;
    my $d_rscript = shift;
    my $graphVizPath = shift;
    my $rh_datasets = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;      # this variable equals $rh_subnets_nwa if we are not in the metasubnetwork case.
    my $rh_noa = shift;
    my $rh_eda = shift;
    my $rh_GO = shift;
    my $menu = shift;
    my $sortByScore = shift;
    my $noHeatmap = shift;
    my $resHeading = shift;
    my $noImage = shift;

    # \%rh_datasets($dataset_label => \%('cond'          => \%($sample_ID => $condition),
    #                                    'rancond'       => \%($sample_ID => $condition),
    #                                    'data'          => \%($probe_ID => \%($sample_ID => $expression)),
    #                                    'collapseddata' => \%($gene_ID => \%($sample_ID => $expression)),
    #                                    'expe'          => \@sample_list,
    #                                    'annot'         => $GEO_platform_ref,
    #                                    'platformType'  => $probe_ID_type_of_platform))
    # \%rh_subnets_nwa($subnetwork_ID => \%('Edges' => \@("$gene_A-$gene_B"),
    #                                       'Nodes' => \%($gene_ID => 1),
    #                                       'Scores' => \%($dataset_label => $subnetwork_score),
    #                                       'p-values' => \%("$dist_head-$dataset_label" => $subnetwork_p_value)),
    #                                    \%('GO_terms' => \%($GO_ID => \%('hyper' => $hypergeometric_p_val,
    #                                                                     'corrected_p_val' => $corrected_p_val,
    #                                                                     'occurrences' => $snw_GO_occurrence))),
    
    # \%rh_noa($gene_ID => \%('Correlations' => \%($dataset_label => $gene_score),
    #                         'Gene_symbol' => $gene_symbol,
    #                         'TF' => 1/undef,
    #                         'Gene_frequency' => $gene_frequency,
    #                         'Best_subnetwork_average_score' => $best_snw_average_score,
    #                         'Gene_degree' => $gene_degree),
    
    # \%rh_eda("$gene_A-$gene_B" => \%($interaction_type => \%('Direction' => $direction_symbol,
    #                                                          'Sources' => $origins)))
    # \%rh_GO($GO_ID => \%('term' => $GO_term,
    #                      'genes' => \@genes_with_GO,
    #                      'subnets' => \@snws_with_GO,
    #                      'network_occurrences' => $total_GO_occurrence))

    # subnet part

    my @a_snws = keys %$rh_subnets_nwa;
    my $level = 2;

    if(scalar(@a_snws) == 0){
        my $subnetPageContent = "";
        $subnetPageContent  = header($subnetPageContent, $projName, $studyname, $level, $network, "subnet", $menu, $resHeading);
        $subnetPageContent .= "<h2>No subnetwork</h2>\n";
        $subnetPageContent  = footer($subnetPageContent);

        my $subnetPageName = "$studyname-$network-subnet.html";
        open (SUBNET, ">$outputDir/$studyname/$network/$subnetPageName") || die "Cannot open $subnetPageName:\n$!";
        print SUBNET $subnetPageContent;
        close SUBNET;

        my $genePageContent = "";
        $genePageContent  = header($genePageContent, $projName, $studyname, $level, $network, "gene", $menu, $resHeading);
        $genePageContent .= "<h2>No gene</h2>\n";
        $genePageContent  = footer($genePageContent);

        my $genePageName = "$studyname-$network-gene.html";
        open (GENE, ">$outputDir/$studyname/$network/$genePageName") || die "Cannot open $genePageName:\n$!";
        print GENE $genePageContent;
        close GENE;

        return;
    }

    my @a_datasets = sort keys %$rh_datasets;
    
    my @a_genes = keys %$rh_noa;

    my @a_values;
    for my $dataset(@a_datasets){
        for my $gene(@a_genes){
            my $val = $$rh_noa{$gene}{'Correlations'}{$dataset};
            push(@a_values, abs($val)) if(defined $val);
        }
        $$rh_datasets{$dataset}{'Gradient_max'} = colorGradient(\@a_values);
    }

    if(defined $$rh_noa{$a_genes[0]}{'Highlights'}){
        my %h_hlGenes;
        for my $geneID(@a_genes){
            for my $k(keys %{$$rh_noa{$geneID}{'Highlights'}}){
                $h_hlGenes{$geneID} = 1 if(defined $$rh_noa{$geneID}{'Highlights'}{$k});
            }
        }
        for my $snwID(keys %$rh_subnets_nwa){
            my $nb = 0;
            for my $geneID(keys %{$$rh_subnets_nwa{$snwID}{'Nodes'}}){
                $nb++ if(defined $h_hlGenes{$geneID});
            }
            $$rh_subnets_nwa{$snwID}{'Highlighted_genes'} = $nb;
        }

        if($network eq 'itg'){
            for my $snwID(keys %$rh_subnetsOrMetas_nwa){
                my $nb = 0;
                for my $geneID(keys %{$$rh_subnetsOrMetas_nwa{$snwID}{'Nodes'}}){
                    $nb++ if(defined $h_hlGenes{$geneID});
                }
                $$rh_subnetsOrMetas_nwa{$snwID}{'Highlighted_genes'} = $nb;
            }
        }
    }

    # %h_dsSuperH('Transcriptome' => \%rh_datasets,
    
    my %h_dsSuperH;
    $h_dsSuperH{'Transcriptome'} = $rh_datasets;
    
    # %h_dsLists('Transcriptome' => \@dataset_labels,
    
    my %h_dsLists;
    $h_dsLists{'Transcriptome'} = \@a_datasets;
    

    my $subnetDir = "$outputDir/$studyname/$network/subnets";
    mkdir $subnetDir;
    my $imageDir = "$outputDir/$studyname/$network/images";
    mkdir $imageDir;

    my $subnetPageContent = "";
    $subnetPageContent = subnetListPage($outputDir,  $itiPath,  $d_rscript,   $graphVizPath, $subnetPageContent, \%h_dsSuperH,
                                        \%h_dsLists, $projName, $studyname, $network,      $rh_subnets_nwa,    $rh_subnetsOrMetas_nwa,
                                        $rh_noa,     $rh_eda,    $menu,         $resHeading,        $rh_GO,
                                        $level,      $sortByScore, $noImage, $noHeatmap);

    my $subnetPageName = "$studyname-$network-subnet.html";
    open (SUBNET, ">$outputDir/$studyname/$network/$subnetPageName") || die "Cannot open $subnetPageName:\n$!";
    print SUBNET $subnetPageContent;
    close SUBNET;


    # gene part

    my $geneDir = "$outputDir/$studyname/$network/genes";
    mkdir $geneDir;

    my $genePageContent = "";
    my $genePageName = "$studyname-$network-gene.html";
    $genePageContent = geneListPage($outputDir,      $genePageContent,       $projName, $studyname,$network, \%h_dsLists,
                                    $rh_subnets_nwa, $rh_subnetsOrMetas_nwa, $rh_noa,    $menu,    $resHeading,
                                    $rh_GO,          $level);

    open (GENE, ">$outputDir/$studyname/$network/$genePageName") || die "Cannot open $genePageName:\n$!";
    print GENE $genePageContent;
    close GENE;

    # GO part

    if(defined $rh_GO){
        my $goDir = "$outputDir/$studyname/$network/GO";
        mkdir $goDir;

        my $goPageContent = "";
        my $goPageName = "$studyname-$network-go.html";
        $goPageContent = goListPage($outputDir,      $goPageContent,        $projName, $studyname, $network, \%h_dsLists,
                                    $rh_subnets_nwa, $rh_subnetsOrMetas_nwa, $rh_noa,    $menu,    $resHeading,
                                    $rh_GO,          $level);

        open (GO, ">$outputDir/$studyname/$network/$goPageName") || die "Cannot open $goPageName:\n$!";
        print GO $goPageContent;
        close GO;
    }

    #print $network." OK.\n";
}

## SUBNETS ##

=pod

=head2 colorGradient()

Calculates the maximum color intensity that nodes can reach in subnetwork representations.

=cut

sub colorGradient {
    my $ra_values = shift;

    @$ra_values = sort{$a <=> $b} @$ra_values;

    # Best gene score apart from the 10 best %.
    my $gradMax = $$ra_values[sprintf("%.0f",(0.9*($#$ra_values)))];

    return $gradMax;
}

=pod

=head2 subnetListPage()

Creates subnetwork pages, tables and images.

=cut

sub subnetListPage {
    my $outputDir = shift;
    my $itiPath = shift;
    my $d_rscript = shift;
    my $graphVizPath = shift;
    my $fileContent = shift;
    my $rh_dsSuperH = shift;
    my $rh_dsLists = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa = shift;
    my $rh_eda = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;
    my $sortByScore = shift;
    my $noImage = shift;
    my $noHeatmap = shift;

    my $tab = "subnet";

    $fileContent = header($fileContent, $projName, $studyname, $level, $network, $tab, $menu, $resHeading, $rh_GO);

    if($network eq "itg"){
        metasubnetImage ($outputDir, $graphVizPath, $studyname, $network, $$rh_dsLists{'Transcriptome'}, $rh_subnets_nwa, $rh_subnetsOrMetas_nwa);

        $fileContent = metasubnetBox($fileContent, $studyname, $network, $level);
        $fileContent = metasubnetListBox($fileContent, $studyname, $network, $rh_subnetsOrMetas_nwa, $rh_noa, $level);
    }
    else {
        $fileContent = subnetListBox($fileContent, $studyname, $network, $rh_dsLists, $rh_subnets_nwa,
                                     $rh_subnetsOrMetas_nwa, $rh_noa, $level, undef);
    }
    $fileContent = footer($fileContent);

    foreach my $subnetID (keys %$rh_subnetsOrMetas_nwa) {
        subnetPage($outputDir,   $itiPath,   $d_rscript,    $graphVizPath, $projName, $studyname,      $network,
                   $rh_dsSuperH, $rh_dsLists, $subnetID,     $rh_subnets_nwa, $rh_subnetsOrMetas_nwa,
                   $rh_noa,      $rh_eda,     $menu,         $resHeading,     $rh_GO,
                   $level, $sortByScore, $noImage, $noHeatmap);
    }

    return $fileContent;
}

=pod

=head2 subnetListBox()

Creates the table listing subnetworks and their information.

=cut

sub subnetListBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa = shift;
    my $level = shift;
    my $goID = shift; ## defined if subnetlist from go page


    my $prefix = '../' x $level;

    my @a_snws = keys %$rh_subnetsOrMetas_nwa;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Subnetworks (".scalar(keys %{$rh_subnetsOrMetas_nwa}).")</h2>\n";

    $fileContent .= "<table class=\"sortable\" >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Subnetwork ID</th>\n";

    for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
        $fileContent .= "<th><i class=\"icon-sort\"></i> $dataset score</th>\n";
    }
    
    if(exists $$rh_subnets_nwa{$a_snws[0]}{'Merged_snw_best_score'}){
        $fileContent .= "<th><i class=\"icon-sort\"></i> Merged snw best score</th>\n";
    }

    if(defined $$rh_subnets_nwa{$a_snws[0]}{'Seed_occurrence'}){
        $fileContent .= "<th><i class=\"icon-sort\"></i> Seed occurrence</th>\n";
    }
    $fileContent .= "<th><i class=\"icon-sort\"></i> Size</th>\n";

    if(defined $$rh_subnets_nwa{$a_snws[0]}{'Highlighted_genes'}){
        my @a_genes = keys %$rh_noa;
        my @a_high  = keys %{$$rh_noa{$a_genes[0]}{'Highlights'}};
        my $head    = "Highlighted genes";
        if(scalar(@a_high) == 1){
            $head = $a_high[0];
            $head =~ s/_/ /g;
        }

        $fileContent .= "<th><i class=\"icon-sort\"></i> $head</th>\n";
    }

    if(defined $goID){
        $fileContent .= "<th><i class=\"icon-sort\"></i> Hypergeometric test</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Bonferroni corrected p-value</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Enrichment ratio</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Occurrences in subnetwork</th>\n";
    }
    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";

    foreach my $subnetID (@a_snws) {

        $fileContent .= "<tr>\n";
        $fileContent .= "<td><a href=\"$prefix$studyname/$network/subnets/$studyname-$network-subnet-$subnetID.html#subnet\">$subnetID</a></td>\n";

        my $score;
        for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){

            $score = $$rh_subnets_nwa{$subnetID}{'Scores'}{$dataset};
            if(defined $score){
                $score = sprintf("%.3f", $score);
            }
            else{
                $score = "-";
            }

            $fileContent .= "<td>$score</td>\n";
        }

        if(defined $$rh_subnets_nwa{$subnetID}{'Merged_snw_best_score'}){
            $score = $$rh_subnets_nwa{$subnetID}{'Merged_snw_best_score'};
            if(defined $score){
                $score = sprintf("%.3f", $score);
            }
            else{
                $score = "-";
            }
            $fileContent .= "<td>$score</td>\n";
        }

        if(defined $$rh_subnets_nwa{$subnetID}{'Seed_occurrence'}){
            $fileContent .= "<td>$$rh_subnets_nwa{$subnetID}{'Seed_occurrence'}</td>\n";
        }
        my $size = scalar(keys(%{$$rh_subnets_nwa{$subnetID}{'Nodes'}}));
        $fileContent .= "<td>$size</td>\n";

        if(defined $$rh_subnets_nwa{$subnetID}{'Highlighted_genes'}){
            $fileContent .= "<td>$$rh_subnets_nwa{$subnetID}{'Highlighted_genes'}</td>\n";
        }

        if(defined $goID){
            my $hyper = $$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$goID}{'hyper'};
            my $corrpval = $$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$goID}{'corrected_pval'};
            my $enrichRatio = $$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$goID}{'Enrichment_ratio'};
            my $occ = $$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$goID}{'occurrences'};
            $fileContent .= "<td>$hyper</td>\n";
            $fileContent .= "<td>$corrpval</td>\n";
            $fileContent .= "<td>$enrichRatio</td>\n";
            $fileContent .= "<td>$occ</td>\n";
        }
        $fileContent .= "</tr>\n";

    }
    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";

    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 metasubnetListBox()

Creates the table listing meta-subnetworks and their information.

=cut

sub metasubnetListBox {
    my $fileContent = shift;
    # my $rh_dsLists = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_meta_nwa = shift;
    my $rh_noa = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    my @a_snws = keys %$rh_meta_nwa;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>".scalar(keys %{$rh_meta_nwa})." meta-subnetworks</h2>\n";

    $fileContent .= "<table class=\"sortable\" >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Subnetwork ID</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Size</th>\n";

    if(defined $$rh_meta_nwa{$a_snws[0]}{'Highlighted_genes'}){
        my @a_genes = keys %$rh_noa;
        my @a_high  = keys %{$$rh_noa{$a_genes[0]}{'Highlights'}};
        my $head    = "Highlighted genes";
        if(scalar(@a_high) == 1){
            $head = $a_high[0];
            $head =~ s/_/ /g;
        }

        $fileContent .= "<th><i class=\"icon-sort\"></i> $head</th>\n";
    }

    # for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
    #     $fileContent .= "<th><i class=\"icon-sort\"></i> $dataset score</th>\n";
    # }

    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";
    foreach my $subnetID (@a_snws) {
        my $size = scalar(@{$$rh_meta_nwa{$subnetID}{'Interactors'}});
        $fileContent .= "<tr>
<td><a href=\"$prefix$studyname/$network/subnets/$studyname-$network-subnet-$subnetID.html#subnet\">$subnetID</a></td>
<td>$size</td>\n";

        if(defined $$rh_meta_nwa{$subnetID}{'Highlighted_genes'}){
            $fileContent .= "<td>$$rh_meta_nwa{$subnetID}{'Highlighted_genes'}</td>\n";
        }

        # my $score;
        # for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
        #     $score = $$rh_meta_nwa{$subnetID}{'Scores'}{$dataset};
        #     if ($network ne "itg" and defined $score){
        #         $score = sprintf("%.3f", $score);
        #     }else{
        #         $score = "-";
        #     }
        #     $fileContent .= "<td>$score</td>\n";
        # }
	
        $fileContent .= "</tr>\n";
    }

    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";

    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 subnetPage()

Creates the page of the specified subnetwork.

=cut

sub subnetPage {
    my $outputDir = shift;
    my $itiPath = shift;
    my $d_rscript = shift;
    my $graphVizPath = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsSuperH = shift;
    my $rh_dsLists = shift;
    my $subnetID = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa = shift;
    my $rh_eda = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;
    my $sortByScore = shift;
    my $noImage = shift;
    my $noHeatmap = shift;

    $level++;
    my $tab = "subnet";

    my @geneList = keys %{$$rh_subnetsOrMetas_nwa{$subnetID}{'Nodes'}};
    my @edgeList = @{$$rh_subnetsOrMetas_nwa{$subnetID}{'Edges'}};

    # heatmap image generation

    unless(defined $noImage){
        subnetImage($outputDir,                    $graphVizPath,          $studyname, $network, $$rh_dsSuperH{'Transcriptome'},
                    $$rh_dsLists{'Transcriptome'}, $rh_subnetsOrMetas_nwa, $rh_noa,    $rh_eda,  $subnetID);

    }

    unless(defined $noHeatmap){
        generateHeatmap($outputDir, $itiPath, $d_rscript, $studyname, $network, $$rh_dsSuperH{'Transcriptome'}, $rh_subnetsOrMetas_nwa, $rh_noa,  $subnetID, undef, $sortByScore);
    }

    # subnet page content

    my $subnetDir = "$outputDir/$studyname/$network/subnets";

    my $fileContent = "";
    my $filename = "$studyname-$network-subnet-$subnetID.html";

    $fileContent = header($fileContent, $projName, $studyname, $level,
                          $network,     $tab,       $menu,
                          $resHeading,  $rh_GO);

    $fileContent .= "<div class=\"title\">\n";
    $fileContent .= "<h1>".$subnetID."</h1>\n";
    $fileContent .= "</div>\n";

    if($network eq "itg"){
        $fileContent = metaSubBox($fileContent, $studyname, $network, $subnetID, $rh_dsLists,
                                  $rh_subnets_nwa, $rh_subnetsOrMetas_nwa, $rh_noa, $level);
    }
    else {
        $fileContent = pvalBox($fileContent, $subnetID, $rh_dsLists, $rh_subnetsOrMetas_nwa);
    }

    $fileContent = subnetBox($fileContent, $rh_dsSuperH,
                             $studyname, $network, $subnetID,
                             $level);
    $fileContent = geneListBox($fileContent, $studyname,
                               $network, $rh_dsLists, \@geneList,
                               $rh_noa, $subnetID, $rh_subnets_nwa, $level);
    $fileContent = interactionListBox($fileContent, $studyname,
                                      $network, \@edgeList, $rh_noa,
                                      $rh_eda, $level);
    if(defined $rh_GO){
        my @goList = sort{$$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$a}{'hyper'} <=> $$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}{$b}{'hyper'}}
                     keys(%{$$rh_subnetsOrMetas_nwa{$subnetID}{'GO_terms'}});
        $fileContent = goListBox($fileContent, $studyname,             $network,   \@goList,
                                 $rh_GO,       $rh_subnetsOrMetas_nwa, $subnetID,  $level,
                                 "subnetpage");
    }

    $fileContent = footer($fileContent);

    # page writing

    open (FILE, ">$subnetDir/$filename") || die "Cannot open $filename:\n$!";
    print FILE $fileContent;
    close FILE;
}

=pod

=head2 pvalBox()

Creates the table containing statistics (score and p-values) of the specified subnetwork.

=cut

sub pvalBox {
    my $fileContent = shift;
    my $subnetID = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Statistics</h2>\n";

    $fileContent .= "<table >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th>Dataset</th>\n";
    $fileContent .= "<th>Score</th>\n";

    my $dataset = $$rh_dsLists{'Transcriptome'}[0];
    my @a_pValHeads;
    @a_pValHeads = keys %{$$rh_subnets_nwa{$subnetID}{'p-values'}} if(defined $$rh_subnets_nwa{$subnetID}{'p-values'});
    my @a_dsPValHeads = sort(grep(/-$dataset$/, @a_pValHeads));
    for my $dist(@a_dsPValHeads){
        $dist =~ s/-.*$//;
        $dist =~ s/dist//;
        $fileContent .= "<th>p-value $dist</th>\n";
    }

    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";

    for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
        $fileContent .= "<tr>\n";
        $fileContent .= "<td>$dataset</td>\n";

        my $score = $$rh_subnets_nwa{$subnetID}{'Scores'}{$dataset};
        if(defined $score){
            $score = sprintf("%.3f", $score);
        }
        else{
            $score = "-";
        }
        $fileContent .= "<td>$score</td>\n";

        my @a_datasetPVals = sort(grep(/-$dataset$/, @a_pValHeads));
        foreach my $p (@a_datasetPVals){
            my $pval = $$rh_subnets_nwa{$subnetID}{'p-values'}{$p};
            if(defined $pval){
                $pval = sprintf("%.2e", $pval);
            }
            else{
                $pval = "-";
            }
            $fileContent .= "<td>$pval</td>\n";
        }
        $fileContent .= "</tr>\n";
    }
    
    
    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";
    $fileContent .= "</div>\n";

    return($fileContent);
}

=pod

=head2 metaSubBox()

Creates the table containing statistics (score and p-values) of the specified meta-subnetwork.

=cut

sub metaSubBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $metaID = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;
    my $rh_meta_nwa = shift;
    my $rh_noa = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Interacting subnets</h2>\n";

    $fileContent .= "<table >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th>Subnetwork</th>\n";
    $fileContent .= "<th>Dataset</th>\n";
    $fileContent .= "<th>Score</th>\n";

    my $dataset = $$rh_dsLists{'Transcriptome'}[0];
    my @a_snws = @{$$rh_meta_nwa{$metaID}{'Interactors'}};
    my @a_pValHeads = keys %{$$rh_subnets_nwa{$a_snws[0]}{'p-values'}};
    my @a_dsPValHeads = sort(grep(/-$dataset$/, @a_pValHeads));
    for my $dist(@a_dsPValHeads){
        $dist =~ s/-.*$//;
        $dist =~ s/dist//;
        $fileContent .= "<th>p-value $dist</th>\n";
    }

    $fileContent .= "<th>Size</th>\n";

    if(defined $$rh_subnets_nwa{$a_snws[0]}{'Highlighted_genes'}){
        my @a_genes = keys %$rh_noa;
        my @a_high  = keys %{$$rh_noa{$a_genes[0]}{'Highlights'}};
        my $head    = "Highlighted genes";
        if(scalar(@a_high) == 1){
            $head = $a_high[0];
            $head =~ s/_/ /g;
        }

        $fileContent .= "<th><i class=\"icon-sort\"></i> $head</th>\n";
    }

    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";

    foreach my $subnetID (@a_snws){

        my $netlink;
        if ($network eq 'itg') { $netlink = $$rh_subnets_nwa{$subnetID}{'Type'}; }
        else { $netlink = $network; }

        my $size = scalar(keys(%{$$rh_subnets_nwa{$subnetID}{'Nodes'}}));
        my $annot;
        $annot = $$rh_subnets_nwa{$subnetID}{'Highlighted_genes'}
            if(defined $$rh_subnets_nwa{$subnetID}{'Highlighted_genes'});

        my @a_pValHeads = keys %{$$rh_subnets_nwa{$subnetID}{'p-values'}};
        for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
            my $score = $$rh_subnets_nwa{$subnetID}{'Scores'}{$dataset};
            if(defined $score){
                $score = sprintf("%.3f", $score);
            }
            else{
                $score = "-";
            }

            $fileContent .= "<tr>\n";
            $fileContent .= "<td><a href=\"$prefix$studyname/$netlink/subnets/$studyname-$netlink-subnet-$subnetID.html#subnet\">$subnetID</a></td>\n";
            $fileContent .= "<td>$dataset</td>\n";
            $fileContent .= "<td>$score</td>\n";

            my @a_datasetPVals = sort(grep(/-$dataset$/, @a_pValHeads));
            foreach my $p (@a_datasetPVals){
                my $pval = $$rh_subnets_nwa{$subnetID}{'p-values'}{$p};
                if(defined $pval){
                    $pval = sprintf("%.2e", $pval);
                }
                else{
                    $pval = "-";
                }
                $fileContent .= "<td>$pval</td>\n";
            }

            $fileContent .= "<td>$size</td>\n";
            $fileContent .= "<td>$annot</td>\n" if(defined $annot);

            $fileContent .= "</tr>\n";
        }
    }

    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";
    $fileContent .= "</div>\n";

    return($fileContent);
}

=pod

=head2 subnetBox()

Creates the images links of the specified subnetwork.

=cut

sub subnetBox {
    my $fileContent = shift;
    my $rh_dsSuperH = shift;
    my $studyname = shift;
    my $network = shift;
    my $subnetID = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    for my $dataset(sort(keys %{$$rh_dsSuperH{'Transcriptome'}})){
        $fileContent .= "<div class=\"box\">\n";
        $fileContent .= "<img class=\"caption\" title=\"$subnetID subnetwork with $dataset dataset\" src=\"$prefix$studyname/$network/images/subnetwork-$network-$subnetID-$dataset.png\" alt=\"$dataset-$subnetID subnetwork\"/>";
        unless(defined $$rh_dsSuperH{'Transcriptome'}{$dataset}{'noHeatmap'}){
            $fileContent .= "<img class=\"caption\" title=\"$subnetID heatmap from $dataset dataset\" src=\"$prefix$studyname/$network/images/heatmap-$network-$subnetID-$dataset.png\" alt=\"$dataset-$subnetID heatmap\"/>";
        }
        $fileContent .= "</div>\n";
    }
    return($fileContent);
}

=pod

=head2 generateHeatmap()

Creates heatmaps of the specified subnetwork genes.

=cut

sub generateHeatmap {
    my $outputDir = shift;
    my $itiPath = shift;
    my $d_rscript = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_datasets = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa = shift;
    my $subnetID = shift;
    my $sortByScore = shift;

    #print "Creating $subnetID heatmap(s)...\n";
    my $imageDir = "$outputDir/$studyname/$network/images";

    foreach my $dataset(keys %$rh_datasets) {
        if(defined $$rh_datasets{$dataset}{'noHeatmap'}){
            #print "No heatmap for dataset $dataset\n";
            next;
        }

        my $expname = "heatmap-$network-$subnetID-$dataset.txt";
        my ($pngname, $scKey, $dsKey);
        
        $scKey = 'Correlations'; $dsKey = $dataset; $pngname = "heatmap-$network-$subnetID-$dataset.png";

        open (EXP, ">$imageDir/$expname") || die "Cannot open $expname:\n$!";

        my @a_samples;

        # In the case cond file is provided, samples are sorted by conditions
        if(scalar(keys %{$$rh_datasets{$dataset}{'cond'}})) {
            @a_samples = sort {$$rh_datasets{$dataset}{'cond'}{$a} cmp $$rh_datasets{$dataset}{'cond'}{$b}}
                              keys %{$$rh_datasets{$dataset}{'cond'}};
        }
        else {
            @a_samples = @{$$rh_datasets{$dataset}{'expe'}};
        }

        my $i = 0;
        foreach my $expe (@a_samples){
            ($i > 0) ? print EXP "\t$expe" : print EXP "$expe";
            $i++;
        }
        print EXP "\n";

        my $condNb = 0;
        if(scalar(keys %{$$rh_datasets{$dataset}{'cond'}})) {
            print EXP "Condition";
            foreach my $expe (@a_samples){
                print EXP "\t$$rh_datasets{$dataset}{'cond'}{$expe}";
            }
            print EXP "\n";
            $condNb = 1;
        }

        elsif(scalar(keys %{$$rh_datasets{$dataset}{'mcond'}})) {
            my @a_condNames = @{$$rh_datasets{$dataset}{'condNames'}};

            for my $cond(@a_condNames){
                print EXP $cond;

                for my $expe(@a_samples){
                    if(defined $$rh_datasets{$dataset}{'mcond'}{$expe}{$cond}){
                        print EXP "\t$$rh_datasets{$dataset}{'mcond'}{$expe}{$cond}";
                    }
                    else{
                        print EXP "\tUNDEF";
                    }
                }

                print EXP "\n";
            }

            $condNb = scalar(@a_condNames);
        }

        my @a_geneIDs;

        # Sort by gene score
        if(defined $sortByScore){
            my(%h_geneScores);
            for my $g(keys(%{$$rh_subnetsOrMetas_nwa{$subnetID}{'Nodes'}})){
                if(defined $$rh_noa{$g}{$scKey}{$dsKey}){
                    $h_geneScores{$g} = $$rh_noa{$g}{$scKey}{$dsKey};
                }
                else{
                    $h_geneScores{$g} = 0;
                }
            }
            @a_geneIDs = sort{$h_geneScores{$a} <=> $h_geneScores{$b}} keys %h_geneScores;
        }
        # Sort by gene symbol
        else{
            @a_geneIDs = sort{$$rh_noa{$b}{'Gene_symbol'} cmp $$rh_noa{$a}{'Gene_symbol'}}
                         keys(%{$$rh_subnetsOrMetas_nwa{$subnetID}{'Nodes'}});
        }

        foreach my $geneID (@a_geneIDs){
            print EXP $$rh_noa{$geneID}{'Gene_symbol'};
            foreach my $expe (@a_samples){

                if(defined $$rh_datasets{$dataset}{'collapseddata'}{$geneID}{$expe}){
                    print EXP "\t$$rh_datasets{$dataset}{'collapseddata'}{$geneID}{$expe}";
                }
                else{
                    print EXP "\t0";    # annHeatmap2 does not support 'NAs'
                }
            }
            print EXP "\n";
        }

        close(EXP);
        system($d_rscript . "Rscript $itiPath/R/tableToHeatmap.R $imageDir/$expname $condNb $imageDir/$pngname");
    }
}

=pod

=head2 metasubnetBox()

Creates the link of the meta-subnetworks connections image.

=cut

sub metasubnetBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Integrated network</h2>\n";
    $fileContent .= "<img class=\"caption\" title=\"...\" src=\"$prefix$studyname/$network/images/$network-metasubnet.png\" alt=\"metasubnets\"/>";
    $fileContent .= "</div>\n";

    return($fileContent);
}

=pod

=head2 subnetImage()

Creates the image of the specified subnetwork.

=cut

sub subnetImage {
    my $outputDir = shift;
    my $graphVizPath = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_datasets = shift;
    my $ra_datasets = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa = shift;
    my $rh_eda = shift;
    my $subnetID = shift;
    

    my $imageDir = "$outputDir/$studyname/$network/images";
    my $dottype  = "digraph";

    foreach my $dataset(@$ra_datasets) {

        my $s_DOT = "$dottype G {\n node[style = filled];\n";
        $s_DOT .= "overlap=false\n";
        #$s_DOT .= "splines=true\n";
        #node[style = filled];\n";

        my($dotname, $pngname);
	
	$dotname = "subnetwork-$network-$subnetID-$dataset.dot";
	$pngname = "subnetwork-$network-$subnetID-$dataset.png";

        my $gradMax = $$rh_datasets{$dataset}{'Gradient_max'};

        my @nodes = keys %{$$rh_subnetsOrMetas_nwa{$subnetID}{'Nodes'}};
        my @a_edges = @{$$rh_subnetsOrMetas_nwa{$subnetID}{'Edges'}};

        foreach my $geneID (@nodes){

            my $shape = "\"triangle\"";
            my $color = "\"black\"";
            my $pen = 0.5;
            my $periph = 1;

            my $value;
	    $value = $$rh_noa{$geneID}{'Correlations'}{$dataset};
	    
            my $geneSymbol = $$rh_noa{$geneID}{'Gene_symbol'};

            if (defined $$rh_noa{$geneID}{'TF'}) { $shape = "\"box\""; }
            else                                 { $shape = "\"egg\""; }

            if($network eq "itg"){
                if (Utils::in_array($geneID, $$rh_subnetsOrMetas_nwa{$subnetID}{'Connectors'})){
                    $color = "\"black\"";
                    $pen = 2;
                }elsif (Utils::in_array($geneID, $$rh_subnetsOrMetas_nwa{$subnetID}{'Int_nodes'})){
                    $color = "\"purple\"";
                    $pen = 1;
                }elsif (Utils::in_array($geneID, $$rh_subnetsOrMetas_nwa{$subnetID}{'Reg_nodes'})){
                    $color = "\"blue\"";
                    $pen = 1;
                }
            }
            else{
                if(defined $$rh_subnetsOrMetas_nwa{$subnetID}{'Merged_snw_seeds'}){
                    $pen = 2 if(Utils::in_array($geneID, $$rh_subnetsOrMetas_nwa{$subnetID}{'Merged_snw_seeds'}));
                }
                elsif($$rh_subnetsOrMetas_nwa{$subnetID}{'Seed'} eq $geneID) {
                    $pen = 2;
                }
            }

            my @k;
            @k = keys %{$$rh_noa{$geneID}{'Highlights'}} if(defined $$rh_noa{$geneID}{'Highlights'});
            foreach my $k (@k){
                if (defined $$rh_noa{$geneID}{'Highlights'}{$k}){
                    $periph = 2;
                }
            }
            if(defined $$rh_subnetsOrMetas_nwa{$subnetID}{'Recurrent_nodes'}){
                if(Utils::in_array($geneID, $$rh_subnetsOrMetas_nwa{$subnetID}{'Recurrent_nodes'})){
                    $periph = 2;
                }
                else{
                    $periph = 1;
                }
            }

            my $gradient;
            if (defined $value and $value != 0 and $value =~ /^-?\d+\.?\d*$/ ) {
                if(abs($value) >= $gradMax){
                    $gradient = 255;
                }
                else{
                    $gradient = sprintf('%.0f', abs($value)*(255/$gradMax));
                }

                if($value < 0) {
                    $s_DOT .= "{node [shape=$shape, peripheries=$periph, color=$color, penwidth=$pen, fillcolor = \"#";
                    $s_DOT .= sprintf("%02x",0);
                    $s_DOT .= sprintf("%02x",150);
                    $s_DOT .= sprintf("%02x",0);
                    $s_DOT .= sprintf("%02x",$gradient);
                    $s_DOT .= "\"] \"$geneSymbol\"}\n";
                } elsif($value > 0) {
                    $s_DOT .= "{node [shape=$shape, peripheries=$periph, color=$color, penwidth=$pen, fillcolor = \"#";
                    $s_DOT .= sprintf("%02x",200);
                    $s_DOT .= sprintf("%02x",0);
                    $s_DOT .= sprintf("%02x",0);
                    $s_DOT .= sprintf("%02x",$gradient);
                    $s_DOT .= "\"] \"$geneSymbol\"}\n";
                }
            } else {
                $s_DOT .= "{node [shape=$shape, peripheries=$periph, color=$color, penwidth=$pen, fillcolor = \"#";
                $s_DOT .= sprintf("%02x",255);
                $s_DOT .= sprintf("%02x",255);
                $s_DOT .= sprintf("%02x",255);
                $s_DOT .= "A0\"] \"$geneSymbol\"}\n";
            }
        }

        foreach my $edgeID (@a_edges){
            my @pair = split('-', $edgeID);
            my $geneIDa = $pair[0];
            my $geneIDb = $pair[1];
            my $geneSymbolA = $$rh_noa{$geneIDa}{'Gene_symbol'};
            my $geneSymbolB = $$rh_noa{$geneIDb}{'Gene_symbol'};

            # next if (! defined      $$rh_eda{$edgeID}{'Direction'});
            for my $it(keys %{$$rh_eda{$edgeID}}){
                if( $$rh_eda{$edgeID}{$it}{'Direction'} eq "--" ){
                    $s_DOT .= "subgraph \"$geneSymbolA$geneSymbolB\" {edge [dir=none, color=purple]"."\"$geneSymbolA\" -> \"$geneSymbolB\"}\n";     ## ! if meta
                }
                if( $$rh_eda{$edgeID}{$it}{'Direction'} eq "<>" || $$rh_eda{$edgeID}{$it}{'Direction'} eq ">" ){
                    $s_DOT .= "edge [color=blue]\n";
                    $s_DOT .= "\"$geneSymbolA\" -> \"$geneSymbolB\"\n";
                }
                if( $$rh_eda{$edgeID}{$it}{'Direction'} eq "<>" || $$rh_eda{$edgeID}{$it}{'Direction'} eq "<" ){
                    $s_DOT .= "edge [color=blue]\n";
                    $s_DOT .= "\"$geneSymbolB\" -> \"$geneSymbolA\"\n";
                }
            }
        }
        $s_DOT .= "}\n";

        open (DOT, ">$imageDir/$dotname") || die "Cannot open $dotname:\n$!";
        print DOT $s_DOT;
        close DOT;

        system("$graphVizPath/neato -Tpng $imageDir/$dotname -o $imageDir/$pngname");
    }
}

=pod

=head2 metasubnetImage()

Creates the image of each meta-subnetwork.

=cut

sub metasubnetImage {

    my $outputDir = shift;
    my $graphVizPath = shift;
    my $studyname = shift,
    my $network = shift;
    my $ra_datasets = shift;
    my $rh_subnets_nwa = shift;
    my $rh_meta_nwa = shift;

    my $imageDir = "$outputDir/$studyname/$network/images";
    my $dottype = "graph";
    my $fillcolor = "\"cadetblue3\"";
    my $color = "\"#00808064\"";
    my $dotname = "$network-metasubnet.dot";
    my $pngname = "$network-metasubnet.png";

    my $s_DOT = "$dottype G {\nsplines=true\noverlap=false\nnode[style = filled];\n";

    # print nodes
    foreach my $metaID (keys %$rh_meta_nwa) {
        foreach my $dataset(@$ra_datasets) {

            my $shape = "\"octagon\"";
            my $pen = 1;

            $metaID =~ s/meta-//g;

            my $type = $$rh_subnets_nwa{$metaID}{'Type'};
            if ($type eq "int")     { $color = "\"purple\""; $fillcolor = "\"#9B30FF64\""; }
            elsif($type eq "reg")   { $color = "\"blue\""; $fillcolor = "\"#0000ff64\""; }

            $s_DOT .= "{node [shape=$shape, style=\"filled\", fillcolor=$fillcolor, color=$color, penwidth=$pen] \"$metaID\"}\n";
        }
    }

    # print edges
    foreach my $metaID (keys %$rh_meta_nwa) {
        foreach my $dataset(@$ra_datasets) {
            my $shape = "\"egg\"";
            my $pen = 1;

            my @nodes = @{$$rh_meta_nwa{$metaID}{'Interactors'}};

            $metaID =~ s/meta-//g;

            foreach my $node (@nodes) {

                next if($node eq $metaID);

                my @seed1 = split('-', $metaID);
                my $seed1 = $seed1[-1];

                my @seed2 = split('-', $node);
                my $seed2 = $seed2[-1];

                if ($seed1 < $seed2){
                    my @connect = Metasubnet::overlapCount($metaID, $node, $rh_subnets_nwa, $rh_subnets_nwa);
                    my $connect = scalar(@connect);
                    for(my $i = 0; $i < $connect; $i++) {
                        $s_DOT .= "\"$node\" -- \"$metaID\"\n"
                    }
                }
            }
        }
    }

    $s_DOT .= "}\n";

    open (DOT, ">$imageDir/$dotname") || die "Cannot open $dotname:\n$!";
    print DOT $s_DOT;
    close DOT;

    system("$graphVizPath/neato -Tpng $imageDir/$dotname -o $imageDir/$pngname");
}

## GENES ##

=pod

=head2 geneListPage()

Creates gene pages and tables.

=cut

sub geneListPage {
    my $outputDir = shift;
    my $fileContent = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa= shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;

    my $dataset = $$rh_dsLists{'Transcriptome'}[0];
    my @geneList = sort{abs($$rh_noa{$b}{'Correlations'}{$dataset}) <=> abs($$rh_noa{$a}{'Correlations'}{$dataset})} keys %$rh_noa;

    my $tab = "gene";

    $fileContent = header($fileContent, $projName, $studyname, $level,
                          $network,     $tab,       $menu,
                          $resHeading,  $rh_GO);

    $fileContent = geneListBox($fileContent, $studyname,
                               $network, $rh_dsLists, \@geneList,
                               $rh_noa, undef, undef, $level);

    foreach my $geneID (@geneList) {
        genePage($outputDir,      $projName, $studyname,           $network,    $geneID, $rh_noa,
                 $rh_subnets_nwa, $rh_subnetsOrMetas_nwa, $rh_dsLists, $menu,   $resHeading,
                 $rh_GO,          $level);
    }

    $fileContent = footer($fileContent);

    return $fileContent;
}

=pod

=head2 geneListBox()

Creates the table listing specified genes and their information.

=cut

sub geneListBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsLists = shift;
    my $ra_geneList = shift;
    my $rh_noa = shift;
    my $subnetID = shift;
    my $rh_subnets_nwa = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Genes (".scalar(@$ra_geneList).")</h2>\n";

    $fileContent .= "<table class=\"sortable\" >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Gene Symbol</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Entrez Gene ID</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Frequency</th>\n";
    my $seed;
    my $comb = 0;
    if(defined $rh_subnets_nwa){
        $seed = $$rh_subnets_nwa{$subnetID}{'Seed'};
        $comb = defined $$rh_subnets_nwa{$subnetID}{'Recurrent_nodes'};
        $fileContent .= "<th><i class=\"icon-sort\"></i> Recurrent node</th>\n" if($comb);
    }
    for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
        $fileContent .= "<th><i class=\"icon-sort\"></i> $dataset gene score</th>\n";
    }
    
    $fileContent .= "<th><i class=\"icon-sort\"></i> Best subnetwork score</th>\n" if(defined $$rh_noa{$$ra_geneList[0]}{'Best_subnetwork_average_score'});

    $fileContent .= "<th><i class=\"icon-sort\"></i> Degree</th>\n" if(defined $$rh_noa{$$ra_geneList[0]}{'Gene_degree'});

    my $TF = exists($$rh_noa{$$ra_geneList[0]}{'TF'});
    $fileContent .= "<th><i class=\"icon-sort\"></i> Transcription factor</th>\n" if($TF);

    my @high;
    if (defined $$rh_noa{$$ra_geneList[0]}{'Highlights'}){
        @high = keys %{$$rh_noa{$$ra_geneList[0]}{'Highlights'}};
        # Warning! If we use @high the content is modified.
        map {s/_/ /g; $fileContent .= "<th><i class=\"icon-sort\"></i> $_</th>\n"}
            keys %{$$rh_noa{$$ra_geneList[0]}{'Highlights'}};
    }

    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";
    for my $geneID (@$ra_geneList) {
        my $freq = $$rh_noa{$geneID}{'Gene_frequency'};

        my $sym;
        if((defined $seed and $geneID eq $seed) ||
           ($comb and Utils::in_array($geneID, $$rh_subnets_nwa{$subnetID}{'Merged_snw_seeds'}))){
            $sym = "[ <a href=\"$prefix$studyname/$network/genes/$studyname-$network-gene-$geneID.html#gene\">$$rh_noa{$geneID}{'Gene_symbol'}</a> ]";
        }
        else{
            $sym = "<a href=\"$prefix$studyname/$network/genes/$studyname-$network-gene-$geneID.html#gene\">$$rh_noa{$geneID}{'Gene_symbol'}</a>";
        }

        $fileContent .= "<tr>\n";
        $fileContent .=
"<td>$sym</td>
<td><a href=\"http://www.ncbi.nlm.nih.gov/gene/$geneID\" target=\"_blank\">$geneID</a></td>
<td>$freq</td>";

        if($comb){
            if(Utils::in_array($geneID, $$rh_subnets_nwa{$subnetID}{'Recurrent_nodes'})){
                $fileContent .= "<td>Yes</td>";
            } else{
                $fileContent .= "<td>-</td>";
            }
        }

        for my $dataset(@{$$rh_dsLists{'Transcriptome'}}){
            my $correlation = $$rh_noa{$geneID}{'Correlations'}{$dataset};

            if(defined $correlation){
                $correlation = sprintf("%.3f", $correlation);
            }
            else{
                $correlation = "-";
            }
            if(defined $subnetID and defined $$rh_noa{$geneID}{'Present_in'}{$dataset}{$subnetID}){
                $fileContent .= "<td><strong>$correlation</strong></td>";
            }
            else{
                $fileContent .= "<td>$correlation</td>";
            }
        }
	
        my $score;
        if(defined $$rh_noa{$geneID}{'Best_subnetwork_average_score'}){
            $score = $$rh_noa{$geneID}{'Best_subnetwork_average_score'};
            $score = sprintf("%.3f", $score);
        }

        $fileContent .= "<td>$score</td>" if(defined $score);

        my $degree = $$rh_noa{$geneID}{'Gene_degree'};
        $fileContent .= "<td>$degree</td>" if(defined $degree);

        if($TF){
            if(defined $$rh_noa{$geneID}{'TF'}){
                $fileContent .= "<td>TF</td>";
            }
            else{
                $fileContent .= "<td>-</td>";
            }
        }

        foreach my $h (@high){
            my $annot = $$rh_noa{$geneID}{'Highlights'}{$h};
            if (defined $annot){
                $annot =~ s/; /;<br>/g;
                $fileContent .= "<td>$annot</td>";
            }
            else{
                $fileContent .= "<td>-</td>";
            }
        }

        $fileContent .= "</tr>\n";
    }
    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";

    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 interactionListBox()

Creates the table listing specified interactions and their information.

=cut

sub interactionListBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $ra_interactionList = shift;
    my $rh_noa = shift;
    my $rh_eda = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Interactions (".scalar @$ra_interactionList.")</h2>\n";

    $fileContent .= "<table class=\"sortable\" >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Gene Symbol 1 </th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Entrez Gene ID 1 </th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Gene Symbol 2</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Entrez Gene ID 2</th>\n";

    $fileContent .= "<th><i class=\"icon-sort\"></i> Type</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Direction</th>\n";
    $fileContent .= "<th>Origin databases / Sources</th>\n";
    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";

    foreach my $edgeID (@{$ra_interactionList}) {
        my @a_edge = split('-', $edgeID);
        my $id1 = $a_edge[0];
        my $id2 = $a_edge[1];

        my $sym1 = $$rh_noa{$id1}{'Gene_symbol'};
        my $sym2 = $$rh_noa{$id2}{'Gene_symbol'};

        for my $type(keys %{$$rh_eda{$edgeID}}){
            my $direction = $$rh_eda{$edgeID}{$type}{'Direction'};
            my $source = $$rh_eda{$edgeID}{$type}{'Sources'};
            my @a_source = split('\|\|', $source);

            my @a_srcStr;
            for my $dbS(@a_source){
                my @a_dbS = split('::', $dbS);
                my $db = $a_dbS[0];
                my $so = $a_dbS[1];
                if(defined $so){
                    $so =~ s/;/, /g;
                }
                else{
                    $so = "no annot";
                }
                push(@a_srcStr, "<strong>$db</strong>: $so");
            }
            my $srcStr = join(';<br>', @a_srcStr);

            $fileContent .= "<tr>\n";
            $fileContent .=
"<td><a href=\"$prefix$studyname/$network/genes/$studyname-$network-gene-$id1.html#gene\">".$sym1."</a></td>
<td><a href=\"http://www.ncbi.nlm.nih.gov/gene/$id1\" target=\"_blank\">$id1</a></td>
<td><a href=\"$prefix$studyname/$network/genes/$studyname-$network-gene-$id2.html#gene\">".$sym2."</a></td>
<td><a href=\"http://www.ncbi.nlm.nih.gov/gene/$id2\" target=\"_blank\">$id2</a></td>
<td>$type</td>
<td>$direction</td>
<td>$srcStr</td>\n";

            $fileContent .= "</tr>\n";
        }
    }
    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";

    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 genePage()

Creates the page of the specified gene.

=cut

sub genePage {
    my $outputDir = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $geneID = shift;
    my $rh_noa = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_dsLists = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;

    $level++;
    my $tab = "gene";
    my @geneList = ($geneID);

    my $geneDir = "$outputDir/$studyname/$network/genes";

    my %h_geneSubnets = ();
    foreach my $sub (@{$$rh_noa{$geneID}{'Subnets'}}) {
        $h_geneSubnets{$sub} = $$rh_subnetsOrMetas_nwa{$sub};
    }

    my $fileContent = "";
    my $filename = "$studyname-$network-$tab-$geneID.html";

    $fileContent = header($fileContent, $projName, $studyname,
                          $level, $network, $tab,
                          $menu, $resHeading, $rh_GO);

    $fileContent .= "<div class=\"title\">\n";
    $fileContent .= "<h1>".$$rh_noa{$geneID}{'Gene_symbol'}."</h1>\n";
    $fileContent .= "</div>\n";

    $fileContent = geneListBox($fileContent, $studyname,
                               $network, $rh_dsLists, \@geneList,
                               $rh_noa, undef, undef, $level);
    $fileContent = geneBoxNCBI($fileContent, $geneID, $rh_noa);

    if($network eq "itg"){
        $fileContent = metasubnetListBox($fileContent, $studyname, $network, \%h_geneSubnets, $rh_noa, $level);
    }
    else{
        $fileContent = subnetListBox($fileContent,    $studyname,      $network, $rh_dsLists,
                                     $rh_subnets_nwa, \%h_geneSubnets, $rh_noa,  $level,      undef);
    }

    if(defined $rh_GO && defined $$rh_noa{$geneID}{'GO_terms'}){
        my @goList = @{$$rh_noa{$geneID}{'GO_terms'}};
        $fileContent = goListBox($fileContent, $studyname,
                                 $network, \@goList, $rh_GO,
                                 undef, undef, $level,
                                 "genepage");
    }

    $fileContent = footer($fileContent);

    open (FILE, ">$geneDir/$filename") || die "Cannot open $filename:\n$!";
    print FILE $fileContent;
    close FILE;
}

=pod

=head2 geneBoxNCBI()

Creates the NCBI link of the specified gene.

=cut

sub geneBoxNCBI{
    my $fileContent = shift;
    my $geneID = shift;
    my $rh_noa = shift;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Gene info in the <a href=\"http://www.ncbi.nlm.nih.gov\" target=\"_blank\">NCBI database</a>: <a href=\"http://www.ncbi.nlm.nih.gov/gene/$geneID\" target=\"_blank\">$$rh_noa{$geneID}{'Gene_symbol'}</a></h2>\n";

    $fileContent .= "</div>\n";

    return($fileContent);
}

## GO ##

=pod

=head2 goListPage()

Creates Gene Ontology pages and tables.

=cut

sub goListPage {
    my $outputDir = shift;
    my $fileContent = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $rh_noa= shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;

    my @goList = keys %{$rh_GO};

    my $tab = "go";
    $fileContent = header($fileContent, $projName, $studyname, $level,
                          $network,     $tab,       $menu,
                          $resHeading,  $rh_GO);

    $fileContent = goListBox($fileContent, $studyname,
                             $network,     \@goList,     $rh_GO,
                             undef,        undef,        $level,
                             "golistpage");

    foreach my $goID (@goList) {
        goPage($outputDir,             $projName, $studyname, $network, $rh_dsLists, $rh_subnets_nwa,
               $rh_subnetsOrMetas_nwa, $goID,      $rh_noa,  $menu,       $resHeading,
               $rh_GO,                 $level);
    }

    $fileContent = footer($fileContent);

    return $fileContent;
}

=pod

=head2 goPage()

Creates the page of the specified GO term.

=cut

sub goPage {
    my $outputDir = shift;
    my $projName = shift;
    my $studyname = shift;
    my $network = shift;
    my $rh_dsLists = shift;
    my $rh_subnets_nwa = shift;
    my $rh_subnetsOrMetas_nwa = shift;
    my $goID = shift;
    my $rh_noa = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;
    my $level = shift;


    $level++;
    my $tab = "go";

    my @goList = ($goID);

    my %h_goSubnets = ();
    foreach my $sub (@{$$rh_GO{$goID}{'subnets'}}) {
        $h_goSubnets{$sub} = $$rh_subnetsOrMetas_nwa{$sub};
    }

    my @geneList = @{$$rh_GO{$goID}{'genes'}};

    my $goDir = "$outputDir/$studyname/$network/GO";

    my $fileContent = "";
    my $filename = "$studyname-$network-$tab-$goID.html";

    $fileContent = header($fileContent, $projName, $studyname, $level,
                          $network,     $tab,       $menu,
                          $resHeading,  $rh_GO);

    $fileContent .= "<div class=\"title\">\n";
    $fileContent .= "<h1>".$goID."</h1>\n";
    $fileContent .= "<h1>".$$rh_GO{$goID}{'term'}."</h1>\n";
    $fileContent .= "</div>\n";

    $fileContent = goListBox($fileContent, $studyname, $network, \@goList,
                             $rh_GO,     undef,        undef,      $level,   "gopage");
    $fileContent = AmiGOBox($fileContent, $goID);
    $fileContent = geneListBox($fileContent, $studyname,
                               $network,     $rh_dsLists,  \@geneList,
                               $rh_noa, undef, undef,      $level);
    $fileContent = subnetListBox($fileContent, $studyname,
                                 $network,     $rh_dsLists,  $rh_subnets_nwa,
                                 \%h_goSubnets, $rh_noa, $level,       $goID);

    $fileContent = footer($fileContent);

    open (FILE, ">$goDir/$filename") || die "Cannot open $filename:\n$!";
    print FILE $fileContent;
    close FILE;
}

=pod

=head2 AmiGOBox()

Creates the AmiGO link of the specified GO term.

=cut

sub AmiGOBox{
    my $fileContent = shift;
    my $goID = shift;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Term information on <a href=\"http://amigo.geneontology.org\" target=\"_blank\">AmiGO 2</a>: <a href=\"http://amigo.geneontology.org/amigo/term/$goID\" target=\"_blank\">$goID</a></h2>\n";
    $fileContent .= "</div>\n";

    return($fileContent);
}

=pod

=head2 goListBox()

Creates the table listing specified GO terms and their information.

=cut

sub goListBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $ra_goList = shift;
    my $rh_GO = shift;
    my $rh_subnets_nwa = shift;
    my $subnetID = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>Related GO terms (".scalar(@{$ra_goList}).")</h2>\n";

    $fileContent .= "<p>Bold terms are significant according to the Benjamini-Hochberg correction (FDR = 5%).<br></p>" if(defined $subnetID);

    $fileContent .= "<table class=\"sortable\" >\n";
    $fileContent .= "<thead>\n";
    $fileContent .= "<tr>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Accession number</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Name</th>\n";
    if(defined $subnetID){
        $fileContent .= "<th><i class=\"icon-sort\"></i> Hypergeometric test</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Bonferroni corrected p-value</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Enrichment ratio</th>\n";
        $fileContent .= "<th><i class=\"icon-sort\"></i> Occurrence in subnetwork</th>\n";
    }
    $fileContent .= "<th><i class=\"icon-sort\"></i> Occurrences in all snw genes</th>\n";
    $fileContent .= "<th><i class=\"icon-sort\"></i> Occurrences in all int/reg genes</th>\n";

    $fileContent .= "</tr>\n";
    $fileContent .= "</thead>\n";
    $fileContent .= "<tbody>\n";
    foreach my $goID (@{$ra_goList}) {

        my $term = $$rh_GO{$goID}{'term'};
        my $allSnwOcc = scalar @{$$rh_GO{$goID}{'genes'}};
        my $totOcc = $$rh_GO{$goID}{'network_occurrences'};

        $fileContent .= "<tr>\n";
        $fileContent .= "<td><a href=\"$prefix$studyname/$network/GO/$studyname-$network-go-$goID.html#go\">".$goID."</a></td>";
        if(defined $subnetID){
            if(defined $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'BH_correction'} and
               $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'BH_correction'} eq '*'){
                $term = "<strong>$term</strong>";
            }

            my $hyper = $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'hyper'};
            my $corrPVal = $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'corrected_pval'};
            my $enrichRatio = $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'Enrichment_ratio'};
            my $snwOcc = $$rh_subnets_nwa{$subnetID}{'GO_terms'}{$goID}{'occurrences'};
            $fileContent .= "<td>$term</td><td>$hyper</td><td>$corrPVal</td><td>$enrichRatio</td><td>$snwOcc</td>";
        }
        else{
            $fileContent .= "<td>$term</td>";
        }
        $fileContent .= "<td>$allSnwOcc</td>";
        $fileContent .= "<td>$totOcc</td>";

        $fileContent .= "</tr>\n";
    }
    $fileContent .= "</tbody>\n";
    $fileContent .= "</table>\n";

    $fileContent .= "</div>\n";

    return $fileContent;
}

## HOMEPAGE STUDY ##

=pod

=head2 homePage()

Creates Home page of the specified analysis.

=cut

sub homePage {
    my $outputDir = shift;
    my $studyname = shift;
    my $menu = shift;
    my $projName = shift;
    my $resHeading = shift;
    my $desc = shift;

    my $level = 1;
    my $tab = "";
    my $network = "home";

    my $fileContent = header("", $projName, $studyname, $level,
                             $network,     $tab,       $menu,
                             $resHeading,  undef);

    $fileContent = studyBox($fileContent, $studyname, $desc);

    $fileContent = footer($fileContent);

    my $homePageName = "$studyname-home.html";
    open (HOME, ">$outputDir/$studyname/$homePageName") || die "Cannot open $homePageName:\n$!";
    print HOME $fileContent;
    close HOME;
}

=pod

=head2 studyBox()

Writes the description of the specified analysis.

=cut

sub studyBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $desc = shift;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h2>$studyname analysis</h2>\n";
    $fileContent .= "<p>$desc</p>\n" if(defined $desc);
    $fileContent .= "</div>\n";

    return $fileContent;
}

## DOWNLOAD STUDY ##

=pod

=head2 downloadPage()

Creates the Download page of the specified analysis.

=cut

sub downloadPage {
    my $outputDir = shift;
    my $studyname = shift;
    my $menu = shift;
    my $projName = shift;
    my $resHeading = shift;
    my $desc = shift;

    my $level = 1;
    my $tab = "";
    my $network = "dl";

    my $fileContent = header("", $projName, $studyname, $level,
                             $network,     $tab,       $menu,
                             $resHeading,  undef);

    $fileContent = studyBox($fileContent, $studyname, $desc);
    $fileContent = downloadBox($fileContent, $studyname, $menu,
                               $level);

    $fileContent = footer($fileContent);

    my $downloadPageName = "$studyname-download.html";

    open (DOWNLOAD, ">$outputDir/$studyname/$downloadPageName") || die "Cannot open $downloadPageName:\n$!";
    print DOWNLOAD $fileContent;
    close DOWNLOAD;
}

=pod

=head2 downloadBox()

Creates files links of the specified analysis.

=cut

sub downloadBox {
    my $fileContent = shift;
    my $studyname = shift;
    my $menu = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">";
    $fileContent .= "<h2>Data download</h2>";

    if ($menu eq "itg"){
        $fileContent .= "<ul class=\"icons\">\n";
        $fileContent .= "<li><a href=\"$prefix$studyname/int/$studyname-int-subnet.html#subnet\"> Interactome</a></li>";
        $fileContent .= "</ul>\n";
        $fileContent .= "<table >\n";
        $fileContent .= "<tr><td>Download subnetworks file (*.nnf): </td><td><a href=\"$prefix$studyname/int/downloads/int-subnetworks.nnf\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download subnetwork statistics: </td><td><a href=\"$prefix$studyname/int/downloads/int-subnetworks_attributes-nwa.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download gene list: </td><td><a href=\"$prefix$studyname/int/downloads/int-genes_attributes-noa.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "</table><br/>\n";
        $fileContent .= "<ul class=\"icons\">\n";
        $fileContent .= "<li><a href=\"$prefix$studyname/reg/$studyname-reg-subnet.html#subnet\"> Regulome</a></li>";
        $fileContent .= "</ul>\n";
        $fileContent .= "<table >\n";
        $fileContent .= "<tr><td>Download subnetworks file (*.nnf): </td><td><a href=\"$prefix$studyname/reg/downloads/reg-subnetworks.nnf\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download subnetworks statistics: </td><td><a href=\"$prefix$studyname/reg/downloads/reg-subnetworks_attributes-nwa.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download gene list: </td><td><a href=\"$prefix$studyname/reg/downloads/reg-genes_attributes-noa.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "</table><br/>\n";
        $fileContent .= "<ul class=\"icons\">\n";
        $fileContent .= "<li><a href=\"$prefix$studyname/itg/$studyname-itg-subnet.html#subnet\"> Integrated Analysis - Meta Subnetworks</a></li>";
        $fileContent .= "</ul>\n";
        $fileContent .= "<table >\n";
        $fileContent .= "<tr><td>All results files (meta-subnetworks and their attributes): </td><td><a href=\"$prefix$studyname/itg/downloads/itg-results.zip\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download meta-subnetworks files (one *.nnf by meta-subnetwork): </td><td><a href=\"$prefix$studyname/itg/downloads/metasubnetworks.zip\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download meta-subnetworks information: </td><td><a href=\"$prefix$studyname/itg/downloads/metasubnets.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download subnetworks list: </td><td><a href=\"$prefix$studyname/itg/downloads/subnets.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "<tr><td>Download genes information: </td><td><a href=\"$prefix$studyname/itg/downloads/meta_nodes.txt\" target=\"_blank\"><i class=\"icon-download-alt\"></i></a></td></tr>\n";
        $fileContent .= "</table>\n";
    }
    else{
        $fileContent .= "<table >\n";
        $fileContent .= "<tr><td>All results files (subnetworks and their attributes):</td><td><a href=\"$prefix$studyname/$menu/downloads/$menu-results.zip\" target=\"_blank\">Download All Files</a></td></tr>\n";
        $fileContent .= "<tr><td>Subnetworks files (one *.nnf by subnetwork):</td><td><a href=\"$prefix$studyname/$menu/downloads/$menu-subnetworks.zip\" target=\"_blank\">Download Nested Network Files</a></td></tr>\n";
        $fileContent .= "<tr><td>Subnetworks information:</td><td><a href=\"$prefix$studyname/$menu/downloads/$menu-subnetworks_attributes-nwa.txt\" target=\"_blank\">Download Subnetwork Attributes</a></td></tr>\n";
        $fileContent .= "<tr><td>Genes information:</td><td><a href=\"$prefix$studyname/$menu/downloads/$menu-genes_attributes-noa.txt\" target=\"_blank\">Download Node Attributes</a></td></tr>\n";
        $fileContent .= "</table><br/>\n";
        $fileContent .= "<ul class=\"icons\">\n";

        $fileContent .= "</ul>\n";
    }

    $fileContent .= "</div>";

    return $fileContent;
}

## INDEX PROJECT ##

=pod

=head2 indexPage()

Creates the Index page of the HTML report.

=cut

sub indexPage {
    my $outputDir = shift;
    my $ra_studies = shift;
    my $projName = shift;
    my $resHeading = shift;

    my $level = 0;
    my $network = "";
    my $tab = "";
    my $menu = "";
    my $fileContent = "";
    my $filename = "index.html";

    $fileContent = header($fileContent, $projName, undef, $level,
                          $network,     $tab,  $menu,
                          $resHeading,  undef);

    $fileContent = indexBox($fileContent, $projName, $ra_studies, $level);

    $fileContent = indexfooter($fileContent);

    open (FILE, ">$outputDir/$filename") || die "Cannot open $filename:\n$!";
    print FILE $fileContent;
    close FILE;
}

=pod

=head2 indexBox()

Creates analyses links in the Index page.

=cut

sub indexBox {
    my $fileContent = shift;
    my $projName = shift;
    my $ra_studies = shift;
    my $level = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<div class=\"box\">\n";
    $fileContent .= "<h1>$projName project</h1>\n";
    $fileContent .= "<ul>\n";

    foreach my $studyname (@$ra_studies){
        $fileContent .= "<li><a href=\"$prefix$studyname/$studyname-home.html\">$studyname</a></li>\n";
    }

    $fileContent .= "</ul>\n";
    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 indexfooter()

Creates the footer of the Index page.

=cut

sub indexfooter {
    my $fileContent = shift;

    $fileContent .= "<div class=\"footer\"></div>\n";
    $fileContent .= "<div id=\"footer\">\n";
    $fileContent .= "<hr />\n";
    $fileContent .= "<img class=\"align-center\" src=\"KickStart/css/logo_cibi.png\" alt=\"cibi logo\"/><br/>\n";
    $fileContent .= "<br/>&copy; Copyright 2008-2015 <a href=\"http://cibi.marseille.inserm.fr/\">Cibi</a><br/>All Rights Reserved<br/>This website was built with <a href=\"http://www.99lime.com\">HTML KickStart</a>\n";

    $fileContent .= "</div>\n";
    $fileContent .= "</body>\n";
    $fileContent .= "</html>\n";

    return $fileContent;
}


## LAYOUT ##

=pod

=head2 header()

Creates the standard header of web pages.

=cut

sub header {
    my $fileContent = shift;
    my $title = shift;
    my $studyname = shift;
    my $level = shift;
    my $network = shift;
    my $tab = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $rh_GO = shift;

    my $prefix = '../' x $level;

    $fileContent .= "<!doctype html>\n";
    $fileContent .= "<html>\n";
    $fileContent .= "<head>\n";
    $fileContent .= "<title>$title</title>\n";

    $fileContent .= "<link rel=\"icon\" type=\"image/png\" href=\"$prefix" . "KickStart/css/sub-icon.png\" />\n";

    $fileContent .= "<!-- KICKSTART -->\n";
    $fileContent .= "<script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js\"></script>\n";
    $fileContent .= "<script src=\"$prefix" . "KickStart/js/kickstart.js\"></script>\n";
    $fileContent .= "<link rel=\"stylesheet\" href=\"$prefix" . "KickStart/css/iti.css\" media=\"all\" />\n";

    $fileContent .= "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\" />\n";

    $fileContent .= "</head>\n";

    $fileContent .= "<body>\n";

    if($level > 0){
        $fileContent = menuBar($fileContent, $studyname, $level,
                               $network,     $menu,      $resHeading,
                               $tab,         $rh_GO)
    }

    $fileContent .= "<div id=\"$tab\" class=\"contenu\">\n";

    return $fileContent;
}

=pod

=head2 menuBar()

Creates the menu of web pages.

=cut

sub menuBar {
    my $fileContent = shift;
    my $studyname = shift;
    my $level = shift;
    my $network = shift;
    my $menu = shift;
    my $resHeading = shift;
    my $tab = shift;
    my $rh_GO = shift;

    my $prefix = '../' x $level;

    my ($regclass, $intclass, $itgclass, $homeclass, $dlclass);
    if ($network eq "reg"){
        $regclass = "\"current\"";
        $intclass = "\"\"";
        $itgclass = "\"\"";
        $homeclass = "\"\"";
        $dlclass = "\"\"";
    }elsif ($network eq "int"){
        $regclass = "\"\"";
        $intclass = "\"current\"";
        $itgclass = "\"\"";
        $homeclass = "\"\"";
        $dlclass = "\"\"";
    }elsif ($network eq "dl"){
        $regclass = "\"\"";
        $intclass = "\"\"";
        $itgclass = "\"\"";
        $homeclass = "\"\"";
        $dlclass = "\"current\"";
    }elsif ($network eq "home"){
        $regclass = "\"\"";
        $intclass = "\"\"";
        $itgclass = "\"\"";
        $homeclass = "\"current\"";
        $dlclass = "\"\"";
    }elsif ($network eq "itg"){
        $regclass = "\"\"";
        $intclass = "\"\"";
        $itgclass = "\"current\"";
        $homeclass = "\"\"";
        $dlclass = "\"\"";
    }

    $fileContent .= "<div id=\"menu\">\n";
    $fileContent .= "<h1><a href=\"$prefix" . "index.html\" class=\"button\"><i class=\" icon-align-justify\"></i></a> $studyname</h1>        \n";

    $fileContent .= "<ul class=\"menu\">\n";
    $fileContent .= "<li class=$homeclass><a href=\"$prefix$studyname/$studyname-home.html\"><i class=\"icon-home\"></i></a></li>\n";
    if($menu eq "itg"){
        $fileContent .= "<li class=$intclass><a href=\"$prefix$studyname/int/$studyname-int-subnet.html#subnet\">Interactome</a></li>\n";
        $fileContent .= "<li class=$regclass><a href=\"$prefix$studyname/reg/$studyname-reg-subnet.html#subnet\">Regulome</a></li>\n";
        $fileContent .= "<li class=$itgclass><a href=\"$prefix$studyname/itg/$studyname-itg-subnet.html#subnet\">Integrated analysis</a></li>\n";
    }
    else{
        $fileContent .= "<li class=$intclass><a href=\"$prefix$studyname/$menu/$studyname-$menu-subnet.html#subnet\">$resHeading</a></li>\n";
    }
    $fileContent .= "<li class=$dlclass><a href=\"$prefix$studyname/$studyname-download.html\"><i class=\"icon-download-alt\"></i></a></li>\n";
    $fileContent .= "</ul>\n";

    if ($level > 1) {
        $fileContent = tabsBar($fileContent, $studyname, $network,
                               $prefix,      $tab,       $rh_GO);
    }

    $fileContent .= "</div>\n";

    return $fileContent;
}

=pod

=head2 tabsBar()

Creates tabs of web pages of the specified analysis.

=cut

sub tabsBar {
    my $fileContent = shift;
    my $studyname = shift;
    my $network = shift;
    my $prefix = shift;
    my $tab = shift;
    my $rh_GO = shift;

    ### ! add go tab under condition
    my ($subtab, $genetab, $gotab);
    if ($tab eq "gene"){
        $subtab = "\"$prefix$studyname/$network/$studyname-$network-subnet.html#subnet\"";
        $genetab = "\"#gene\"";
        $gotab = "\"$prefix$studyname/$network/$studyname-$network-go.html#go\"";
    }elsif ($tab eq "subnet"){
        $subtab = "\"#subnet\"";
        $genetab = "\"$prefix$studyname/$network/$studyname-$network-gene.html#gene\"";
        $gotab = "\"$prefix$studyname/$network/$studyname-$network-go.html#go\"";
    }elsif ($tab eq "go"){
        $subtab = "\"$prefix$studyname/$network/$studyname-$network-subnet.html#subnet\"";
        $genetab = "\"$prefix$studyname/$network/$studyname-$network-gene.html#gene\"";
        $gotab = "\"#go\"";
    }

    $fileContent .= "<ul class=\"tabs left\">\n";
    $fileContent .= "<li></li>\n";
    $fileContent .= "<li><a href=$subtab><i class=\"icon-sitemap\"></i> Subnetworks</a></li>\n";
    $fileContent .= "<li><a href=$genetab><i class=\"icon-map-marker\"></i> Genes</a></li>\n";
    if(defined $rh_GO){
        $fileContent .= "<li><a href=$gotab><i class=\"icon-plus\"></i> GO terms</a></li>\n";
    }
    $fileContent .= "</ul>\n";

    return $fileContent;
}

=pod

=head2 footer()

Creates the standard footer of web pages.

=cut

sub footer {
    my $fileContent = shift;

    $fileContent .= "<div class=\"footer\"></div>\n";
    $fileContent .= "<div id=\"footer\">\n";
    $fileContent .= "<hr />&copy; Copyright 2008-2015 <a href=\"http://cibi.marseille.inserm.fr/\">Cibi</a> &ndash; All Rights Reserved &ndash; This website was built with <a href=\"http://www.99lime.com\">HTML KickStart</a>\n";

    $fileContent .= "</div>\n";
    $fileContent .= "</body>\n";
    $fileContent .= "</html>\n";

    return $fileContent;
}

1;
