#!/usr/bin/perl
#
# HTS-Net project
#
# Copyright Maxime Garcia, Ghislain Bidaut, Claire Rioualen, Quentin Da Costa
# (2009-2017)
#
# maxime.u.garcia@gmail.com
# ghislain.bidaut@inserm.fr
# claire.rioualen@inserm.fr
# quentin.da-costa@inserm.fr
#
# http://iti.sourceforge.net
#
# This software is a computer program whose purpose is to superimpose
# interactome over several gene expression datasets to find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 Name
extractAutomatedPValues.pl

=head1 Synopsis

=head1 Examples

=head1 Description

=head1 Subroutines

=head1 Authors
Ghislain Bidaut     ghislain.bidaut@inserm.fr
Quentin Da Costa    quentin.da-costa@inserm.fr
Maxime Garcia       maxime.u.garcia@gmail.com
Claire Rioualen     claire.rioualen@inserm.fr

=cut


## Modules ##
use strict;
use warnings;

use Getopt::Long;
use Utils;

## Arguments management ##


my ($help,
    @a_f_countFiles,
    $f_output
);


GetOptions(
    'help|usage'           => \$help,
    'countFiles|cf=s{,}'   => \@a_f_countFiles,
    'outputFile|of=s'       => \$f_output,
    );


if(defined $f_output and
   scalar @a_f_countFiles) {
    
    
# Read all countfiles and compute p-values max for minimal number of subnetworks
    my $rh_pValues = readCountFiles(\@a_f_countFiles);
    my  @a_pValues;
    
    die "Cannot find suitable p-values\n" unless(scalar (keys %$rh_pValues) > 0);
    foreach my $f (keys %$rh_pValues) {
	push(@a_pValues, $$rh_pValues{$f});
    }
    
    my $pValue = Utils::max(@a_pValues);
    
    print "********** Retained pvalue = $pValue\n";
    
    open(FILE, ">$f_output") || die "Cannot open $f_output: $!";
    print FILE "# P value chosen rom following files:\n";
    foreach (@a_f_countFiles) {
	print FILE "# $_\n";
    }
    print FILE "# retained p-value:\n";
    print FILE "$pValue\n";
    close FILE;
    
}
else {
    print "
NAME
$0

DESCRIPTION

PARAMETERS
";
    exit(0);
}
 
sub readCountFiles {
    
    my $ra_f_countFiles = shift;

    my %h_pValues;

    foreach my $file(@$ra_f_countFiles) {
	
	open(FILE, "$file") || die "Cannot open $file: $!";
	
	my %h_curCount;

	while(defined(my $l = <FILE>)){
	    next if ($l =~ m/^#/ || $l =~ m/^Kept/);
	    
	    $l =~ s/[\r\n]+$//;
	    my @row = split('\t', $l);
	    
	    $h_curCount{$row[1]} = $row[0];
	}
	close FILE;

	foreach my $count (sort  { $a <=> $b } keys %h_curCount) {
	    my $pValue = $h_curCount{$count};
	    	    
	    if($count > 20 && $count < 200) {
		
		$h_pValues{$file} = $pValue;
		
		last;
	    }
	    
	}
    }

    return \%h_pValues;
}
