#!/usr/bin/perl
#
# ITI project
#
# Copyright Maxime Garcia, Ghislain Bidaut, Claire Rioualen, Quentin Da Costa
# (2009-2017)
#
# maxime.u.garcia@gmail.com
# ghislain.bidaut@inserm.fr
# claire.rioualen@inserm.fr
# quentin.da-costa@inserm.fr
#
# http://iti.sourceforge.net
#
# This software is a computer program whose purpose is to superimpose
# interactome over several gene expression datasets to find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 Name
extractSubnetwork.pl

=head1 Examples
perl extractSubnetwork.pl -nnf test.nnf

=head1 Subroutines
=over

=item

=back

=head1 Authors
Ghislain Bidaut     ghislain.bidaut@inserm.fr
Quentin Da Costa    quentin.da-costa@inserm.fr
Maxime Garcia       maxime.u.garcia@gmail.com
Claire Rioualen     claire.rioualen@inserm.fr

=cut


## Modules ##

use strict;
use warnings;
use Data::Dumper;

use Getopt::Long;

use Subnetworks;


## Arguments management ##

my($f_nnf, $d_od, $f_nwa);

GetOptions('networkFile|nnf=s'    => \$f_nnf,
           'networkAttFile|nwa=s' => \$f_nwa,
           'outputDir|od=s'       => \$d_od);

die "\nMissing parameter!\n\n" if(!defined $f_nnf || !defined $d_od);
mkdir $d_od;
my($rh_subnets_nwa, $rh_nodeAtts, $rh_edgeAtts) = Subnetworks::readSubnets($f_nnf, $f_nwa);
$d_od =~ s/\/+$//;

for my $snwID(keys %$rh_subnets_nwa){
    open(NNF, ">$d_od/$snwID.nnf") || die "Cannot open $d_od/$snwID.nnf: $!";
    #print "Writing $snwID subnetwork file\n";
    print NNF "# Network\tProtein_A\tRelationship_type\tProtein_B\n";

    for my $int(@{$$rh_subnets_nwa{$snwID}{'Edges'}}){
        my @a_int = split(/-/, $int);
        my $g1 = $a_int[0];
        my $g2 = $a_int[1];

        for my $intType (keys %{$$rh_edgeAtts{$int}}){
            print NNF "$snwID\t$g1\t$intType\t$g2\n";
        }
    }

    close NNF;

    if(defined $$rh_subnets_nwa{$snwID}{'Recurrent_nodes'} and defined $$rh_subnets_nwa{$snwID}{'Merged_snw_seeds'}){
        open(NOA, ">$d_od/$snwID-noa.txt") || die "Cannot open $d_od/$snwID-noa.txt: $!";
        #print "Writing $snwID noa file\n";
        print NOA "Gene_ID\tRecurrent_node\tMerged_snw_seed\n";

        my %h_snwNoa;
        for my $g(keys %{$$rh_subnets_nwa{$snwID}{'Nodes'}}){
            if(utils::in_array($g, $$rh_subnets_nwa{$snwID}{'Recurrent_nodes'})){
                    print NOA "$g\tRecurrent";
            }
            else{
                    print NOA "$g\t-";
            }
            if(utils::in_array($g, $$rh_subnets_nwa{$snwID}{'Merged_snw_seeds'})){
                    print NOA "\tSeed\n";
            }
            else{
                    print NOA "\t-\n";
            }
        }
        close NOA;
    }
}

#print "\n$0 done.\n\n";
