#!/bin/bash
#
# ITI project
#
# Copyright Garcia Maxime
# contributor(s) : Bidaut Ghislain, Stahl Olivier
# (2009-2010)
# 
# maxime.garcia@inserm.fr
# ghislain.bidaut@inserm.fr
# http://bioinformatique.marseille.inserm.fr/iti/
# 
# This software is a computer program whose purpose is to superimpose
# interactome  over  several  gene  expression  datasets  to  find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

#This script makes threshold files used by the filterSubnetworks script

#DIRECTORIES
directory='FALSE'

#OTHERS
index=0

#PARAMETERS
for PARAM in $@ ; do
	let 'index+=1'
	case $PARAM in
		-h | --help | -u | --usage )
			echo
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tITI project : a computer program whose purpose is to superimpose"
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tinteractome over several gene expression datasets to find"
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tdiscriminative subnetworks predicting metastatic relapse in"
			echo -e "\t       @@@@@       \tbreast cancer."
			echo -e "\t@@@@@  @@@@@  @@@@@\t"
			echo -e "\t@@@@@  @@@@@  @@@@@\tAuthor         : Garcia Maxime"
			echo -e "\t@@@@@  @@@@@  @@@@@\tContributor(s) : Bidaut Ghislain, Stahl Olivier"
			echo -e "\t@@@@@  @@@@@  @@@@@\tLicence        : CeCILL http://www.cecill.info"
			echo -e "\t@@@@@  @@@@@  @@@@@\t"
			echo -e "\t@@@@@  @@@@@  @@@@@\thttp://bioinformatique.marseille.inserm.fr/iti/"
			echo -e "\nNAME\n\t$0 - This script generate Matlab scripts to make classification curves\n"
			echo -e "SYNOPSIS\n\t$0 -d [directory]\n"
			echo -e "DESCRIPTION\n\tmake pictures based on the classification resume file made by the script readClassificationFiles from the ITI project\n"
			echo -e "PARAMETERS\n"
			echo -e "\t-h | --help | -u | --usage\n\t\tDisplay this help and exit\n"
			echo -e "\t-a | --about\n\t\tDisplay author information and exit\n"
			echo -e "\t-v | --version\n\t\tDisplay version information and exit\n"
			echo -e "\t-d | -dir | --directory <directory>\n\t\tUse the given directory as principal directory\n"
			echo -e "Please report every problem at <maxime.garcia@inserm.fr> or <max.u.garcia@gmail.com>"
			exit 0
			;;
		-a | --about )
			echo -e "\tAuthor : Maxime Garcia\n\t<maxime.garcia@inserm.fr> or <max.u.garcia@gmail.com>\n"
			exit 0
			;;
		-v | --version )
			echo -e "\t$0\n\t15/02/2010 version\n"
			exit 0
			;;
		-d | -dir | --directory )
			let 'i=index+1'
			directory=${@:$i:1}
			;;
		$directory )
			;;
		* )
			echo -e "\t$0 Invalid option -- $PARAM\n\tTo know more, do «$0 --help»\n"
			exit 0
			;;
	esac
done

if [[ ! -d $directory ]]
then
	echo "Error: '$directory' does not exist!!"
	echo "type -h for help"
	exit 1
fi

#RAJOUTER UNE FONCTION DE CHECKAGE D'EXISTENCE DU REPERTOIRE /!\ IOP

cd $directory

ls -1 | grep RESUME | grep .txt > TEMP1
sed s/.txt//g TEMP1 > TEMP
rm -f TEMP1

for file in `cat TEMP`;do
	echo "$file.txt in process"
	echo "DELIMITER = '\t';" > script.m
	echo "HEADERLINES = 1;" >> script.m
	echo "newData1 = importdata('$file.txt', DELIMITER, HEADERLINES);" >> script.m
	echo "colheaders = genvarname(newData1.textdata);" >> script.m
	echo "for i = 1:length(colheaders)" >> script.m
	echo "dataByColumn1.(colheaders{i}) = newData1.data(:, i);" >> script.m
	echo "end" >> script.m
	echo "vars = fieldnames(dataByColumn1);" >> script.m
	echo "assignin('base', 'Number', dataByColumn1.(vars{1}) );" >> script.m
	echo "assignin('base', 'Sensibility', dataByColumn1.(vars{2}) );" >> script.m
	echo "assignin('base', 'Specificity', dataByColumn1.(vars{3}) );" >> script.m
	echo "assignin('base', 'Discovery_rate', dataByColumn1.(vars{4}) );" >> script.m
	echo "plot(Number,Sensibility,'XDataSource','Number','YDataSource','Sensibility'); hold all;" >> script.m
	echo "plot(Number,Specificity,'XDataSource','Number','YDataSource','Specificity'); hold all;" >> script.m
	echo "plot(Number,Discovery_rate,'XDataSource','Number','YDataSource','Discovery_rate'); hold all;" >> script.m
	echo "hold off;figure(gcf);" >> script.m
	echo "figure1 = figure('Color',[0.8 0.8 0.8]);" >> script.m
	echo "axes1 = axes('Parent',figure1,'XGrid','on','YGrid','on');" >> script.m
	if [[ $(echo $file | grep GENES | wc -l) == 1 ]]
	then
		i=$(tail -n 1 $file.txt | cut -f 1)
		let 'i+=5'
		echo "xlim(axes1,[0 $i]);" >> script.m
	fi
	echo "ylim(axes1,[0 1]);" >> script.m
	echo "box(axes1,'on');" >> script.m
	echo "hold(axes1,'all');" >> script.m
	echo "YMatrix1 = ([ Sensibility Specificity Discovery_rate ]);" >> script.m
	echo "plot1 = plot(Number,YMatrix1,'Parent',axes1,'LineWidth',2);" >> script.m
	echo "set(plot1(1),'DisplayName','Sensibility');" >> script.m
	echo "set(plot1(2),'DisplayName','Specificity');" >> script.m
	echo "set(plot1(3),'DisplayName','Discovery rate');" >> script.m
	if [[ $(echo $file | grep GENES | wc -l) == 1 ]]
	then
		echo "xlabel('Number of genes','FontWeight','bold','FontSize',14);" >> script.m
	else
		echo "xlabel('Number of subnetworks','FontWeight','bold','FontSize',14);" >> script.m
	fi
	echo "ylabel('Rate','FontWeight','bold','FontSize',14);" >> script.m
	echo "title('$file','FontWeight','bold','FontSize',16);" >> script.m
	echo "legend1 = legend(axes1,'show');" >> script.m
	echo "set(legend1,'Orientation','horizontal','Location','SouthEast');" >> script.m
	echo "set(axes1,'Position',[0.07 0.11 0.9 0.83]);" >> script.m
	echo "print ('-dtiff', '$file.tiff');" >> script.m
	echo "saveas(gcf,'$file.fig');" >> script.m
	/home/opt/bin/matlab -nodisplay -nosplash < script.m
	rm -f script.m
	echo -e "\n\a"
done
i=0
echo "DELIMITER = '\t';" > script.m
echo "HEADERLINES = 1;" >> script.m
for file in `cat TEMP`;do
	let 'i+=1'
	echo "newData$i = importdata('$file.txt', DELIMITER, HEADERLINES);" >> script.m
	echo "colheaders = genvarname(newData$i.textdata);" >> script.m
	echo "for i = 1:length(colheaders)" >> script.m
	echo "dataByColumn$i.(colheaders{i}) = newData$i.data(:, i);" >> script.m
	echo "end" >> script.m
	echo "vars = fieldnames(dataByColumn$i);" >> script.m
	echo "assignin('base', 'Sensibility$i', dataByColumn$i.(vars{2}) );" >> script.m
	echo "assignin('base', 'Specificity$i', dataByColumn$i.(vars{3}) );" >> script.m
	echo "R_Specificity$i=1-Specificity$i;" >> script.m
done
i=0
echo "NONE = [0:0.1:1]';" >> script.m
echo "figure1 = figure('Color',[0.8 0.8 0.8]);" >> script.m
echo "axes1 = axes('Parent',figure1);" >> script.m
echo "box(axes1,'on');" >> script.m
echo "hold(axes1,'all');" >> script.m

sed s/RESUME-//g TEMP > TEMP1
cat TEMP1 > TEMP
rm -f TEMP1

for name in `cat TEMP`;do
	let 'i+=1'
	echo "plot(R_Specificity$i,Sensibility$i,'Parent',axes1,'DisplayName','$name');" >> script.m
done
echo "plot(NONE,NONE,'Parent',axes1,'Color',[0 0 0]);" >> script.m
echo "xlabel('1 - Specificity');" >> script.m
echo "ylabel('Sensibility');" >> script.m
echo "title('ROC Space - RUN-A1xxxx','FontSize',12);" >> script.m
echo "legend1 = legend(axes1,'show');" >> script.m
echo "set(legend1,'Location','SouthEast');" >> script.m
echo "set(axes1,'Position',[0.07 0.07 0.9 0.87]);" >> script.m
echo "name = 'ROC.tiff';" >> script.m
echo "print ('-dtiff', name);" >> script.m
/home/opt/bin/matlab -nodisplay -nosplash < script.m
echo -e "\n\a"

rm -f script.m TEMP
