#!/usr/bin/perl
#
# ITI project
#
# Copyright Maxime Garcia, Ghislain Bidaut, Claire Rioualen, Quentin Da Costa
# (2009-2014)
#
# maxime.u.garcia@gmail.com
# ghislain.bidaut@inserm.fr
# claire.rioualen@inserm.fr
# quentin.da-costa@inserm.fr
#
# http://iti.sourceforge.net
#
# This software is a computer program whose purpose is to superimpose
# interactome over several gene expression datasets to find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

use strict;
use warnings;
use Data::Dumper;
use Getopt::Long;

use Subnetworks;

my($f_nwa, $d_out, $f_nnf);

GetOptions('nwaf=s' => \$f_nwa,
           'nnf=s'  => \$f_nnf,
           'out=s'  => \$d_out);

my($rh_nwa, $rh_noa, $rh_eda) = Subnetworks::readSubnets($f_nnf, $f_nwa);
my @a_subnetIDs = keys %$rh_nwa;
my @a_datasets  = keys %{$$rh_nwa{$a_subnetIDs[0]}{'Scores'}};

for my $ds(@a_datasets){
	my @a_scores;
	for my $subnetID(@a_subnetIDs){
		push(@a_scores, $$rh_nwa{$subnetID}{'Scores'}{$ds}) if(defined $$rh_nwa{$subnetID}{'Scores'}{$ds});
	}
	@a_scores = sort(@a_scores);
	my $coeff = 1 / $a_scores[-1];

	for my $subnetID(@a_subnetIDs){
		$$rh_nwa{$subnetID}{'Scores'}{$ds} = $$rh_nwa{$subnetID}{'Scores'}{$ds} * $coeff if(defined $$rh_nwa{$subnetID}{'Scores'}{$ds});
	}
}

my $d_realScores = "$d_out/real_scores";
mkdir $d_realScores;
rename($f_nwa, "$d_realScores/snw_attributes-nwa.txt");
Subnetworks::writeSubnets($rh_nwa, undef, undef,
                     undef, "$d_out/snw_attributes-norm-nwa.txt", undef);
