#!/bin/bash
#
# ITI project
#
# Copyright Garcia Maxime
# contributor(s) : Bidaut Ghislain, Stahl Olivier
# (2009-2010)
# 
# maxime.garcia@inserm.fr
# ghislain.bidaut@inserm.fr
# http://bioinformatique.marseille.inserm.fr/iti/
# 
# This software is a computer program whose purpose is to superimpose
# interactome  over  several  gene  expression  datasets  to  find
# discriminative subnetworks predicting metastatic relapse in breast
# cancer.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

#This script selects subnetworks for intersectSubnetworks.pl

#DIRECTORIES
directory='FALSE'

#OTHERS
index=0

#PARAMETERS
for PARAM in $@ ; do
	let 'index+=1'
	case $PARAM in
		-h | --help | -u | --usage )
			echo
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tITI project : a computer program whose purpose is to superimpose"
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tinteractome over several gene expression datasets to find"
			echo -e "\t@@@@@@@@@@@@@@@@@@@\tdiscriminative subnetworks predicting metastatic relapse in"
			echo -e "\t       @@@@@       \tbreast cancer."
			echo -e "\t@@@@@  @@@@@  @@@@@\t"
			echo -e "\t@@@@@  @@@@@  @@@@@\tAuthor         : Garcia Maxime"
			echo -e "\t@@@@@  @@@@@  @@@@@\tContributor(s) : Bidaut Ghislain, Stahl Olivier"
			echo -e "\t@@@@@  @@@@@  @@@@@\tLicence        : CeCILL http://www.cecill.info"
			echo -e "\t@@@@@  @@@@@  @@@@@\t"
			echo -e "\t@@@@@  @@@@@  @@@@@\thttp://bioinformatique.marseille.inserm.fr/iti/"
			echo -e "\nNAME\n\t$0 - This script selects subnetworks for intersectSubnetworks.pl from the ITI project\n"
			echo -e "SYNOPSIS\n\t$0 -d [directory] -f [floor] -c [ceil]\n"
			echo -e "DESCRIPTION\n\tSelect subnetworks for intersectSubnetworks.pl from the ITI project\n"
			echo -e "PARAMETERS"
			echo -e "\t-h | --help | -u | --usage\n\t\tDisplay this help and exit\n"
			echo -e "\t-a | --about\n\t\tDisplay author information and exit\n"
			echo -e "\t-v | --version\n\t\tDisplay version information and exit\n"
			echo -e "\t-d | -dir | --directory <directory>\n\t\tUse the given directory as principal directory\n"
			echo -e "\t-f | -floor <number>\n\t\tUse the given number as lower bound for the number of subnets kept\n"
			echo -e "\t-c | -ceil <number>\n\t\tUse the given number as upper bound for the number of subnets kept\n"

			echo -e "Please report every problem at <maxime.garcia@inserm.fr> or <max.u.garcia@gmail.com>"
			exit 0
			;;
		-a | --about )
			echo -e "\tAuthor : Maxime Garcia\n\t<maxime.garcia@inserm.fr> or <max.u.garcia@gmail.com>\n"
			exit 0
			;;
		-v | --version )
			echo -e "\t$0\n\t07/04/2010 version\n"
			exit 0
			;;
		-d | -dir | --directory )
			let 'i=index+1'
			directory=${@:$i:1}
			;;
		-f | -floor )
			let 'i=index+1'
			floor=${@:$i:1}
			;;
		-c | -ceil )
			let 'i=index+1'
			ceil=${@:$i:1}
			;;
		$directory | $ceil | $floor )
			;;
		* )
			echo -e "\t$0 Invalid option -- $PARAM\n\tTo know more, do «$0 --help»\n"
			exit 0
			;;
	esac
done

if [ ! -d $directory ]
then
        echo "Error: '$directory' does not exist!!"
        echo "type -h for help"
        exit 1
fi

cd $directory

ls -1 $directory/lists/ | grep subnets-kept_th1 > subnets1.list
ls -1 $directory/lists/ | grep subnets-kept_th2 > subnets2.list
ls -1 $directory/lists/ | grep subnets-kept_th3 > subnets3.list

for i in `cat subnets1.list`;do

	l=$(cat $directory/lists/$i | grep -v \# | grep subnet.txt | wc -l)
	if [ "$l" -le "$floor" ]
	then
		continue
	else
		echo $i >> subnets1new.list
	fi

done

echo -e "Filtering first list of subnets\nBefore filter : $(cat subnets1.list | wc -l) files\nAfter  filter : $(cat subnets1new.list | wc -l) files"

for i in `cat subnets2.list`;do

	l=$(cat $directory/lists/$i | grep -v \# | grep subnet.txt | wc -l)
	if [ "$l" -le "$floor" ]
	then
		continue
	else
		echo $i >> subnets2new.list
	fi

done

echo -e "Filtering second list of subnets\nBefore filter : $(cat subnets2.list | wc -l) files\nAfter  filter : $(cat subnets2new.list | wc -l) files"

for i in `cat subnets3.list`;do

	l=$(cat $directory/lists/$i | grep -v \# | grep subnet.txt | wc -l)
	if [ "$l" -le "$floor" ]
	then
		continue
	else
		echo $i >> subnets3new.list
	fi

done

echo -e "Filtering third list of subnets\nBefore filter : $(cat subnets3.list | wc -l) files\nAfter  filter : $(cat subnets3new.list | wc -l) files"

for i in `cat subnets1new.list`;do

	for j in `cat subnets2new.list`;do

		for k in `cat subnets3new.list`;do

			rm -rf TEMP*

			echo -e "$i\t$j\t$k"

			cat $directory/lists/$i | grep -v \# | grep subnet.txt > TEMP1
			cat $directory/lists/$j | grep -v \# | grep subnet.txt > TEMP2
			cat $directory/lists/$k | grep -v \# | grep subnet.txt > TEMP3

			touch TEMP

			for l in `cat TEMP2`;do

				m=$(cat TEMP1 | grep $l)

				if [ -n "$m" ]
				then
					echo $m >> TEMP
				fi

			done

			l=$(cat TEMP | wc -l)

			if [ "$l" -le "$floor" ]
			then
				echo -e "skipped -le $floor post intersection 2"
				continue
			fi

			touch TEMP0

			for l in `cat TEMP3`;do

				m=$(cat TEMP | grep $l)

				if [ -n "$m" ]
				then
					echo $m >> TEMP0
				fi

				l=$(cat TEMP0 | sort -u | wc -l)

				if [ "$l" -ge "$ceil" ]
				then
					echo -e "skipped -ge $ceil loop in intersection 3"
					continue 2
				fi

			done

			l=$(cat TEMP0 | sort -u | wc -l)

			if [ "$l" -ge "$floor" ]
			then
				echo -e "$i\t$j\t$k\t$l" >> RESUME
				echo -e "kept"
			else
				echo -e "removed -lt $floor"
			fi

		done

	done

done
